"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const transformer_1 = __importDefault(require("./transformer"));
const micromatch_1 = __importDefault(require("micromatch"));
const crypto_1 = require("crypto");
const middlewareFunction = ({ options, middlewareUtil, resources, log }) => {
    const config = {
        include: ['**/*.js'],
        exclude: [],
        babelConfig: {
            presets: ['@babel/preset-env'],
            sourceMaps: 'inline'
        },
        iife: true,
        searchInclude: ['**/*.js'],
        searchExclude: [],
        onError: 'error',
        ...options.configuration
    };
    const transformer = new transformer_1.default(config.babelConfig, config.iife);
    const reader = middlewareUtil.resourceFactory.createFilterReader({
        reader: resources.all,
        callback: r => micromatch_1.default.isMatch(r.getPath(), config.searchInclude, { ignore: config.searchExclude })
    });
    const writer = middlewareUtil.getProject().getReader()._readers.find((w) => !!w.write);
    return async (req, res, next) => {
        try {
            const path = middlewareUtil.getPathname(req);
            const matched = micromatch_1.default.isMatch(path, config.include, { ignore: config.exclude });
            if (!matched)
                return next();
            const searchString = (0, utils_1.splitPathExtension)(path)[0] + '.*';
            const potentialResources = await reader.byGlob(searchString);
            const resource = potentialResources[0];
            if (!resource)
                return next();
            const contents = await resource.getString();
            const transformed = await transformer.transform(contents, resource.getPath());
            const passId = (0, crypto_1.randomUUID)();
            console.log(passId, path, resource.getPath(), req.path);
            const file = middlewareUtil.resourceFactory.createResource({
                path: `/${passId}+${resource.getPath()}`,
                string: transformed,
                statInfo: resource.getStatInfo()
            });
            await writer.write(file, { drain: true });
            req.path = `/${passId}+${req.path}`;
            next();
        }
        catch (ex) {
            log.error(ex.message || 'unknown error');
            switch (config.onError) {
                case 'next':
                    return next();
                case 'exit':
                    return process.exit(-1);
                default:
                    res.status(503).end();
            }
        }
    };
};
module.exports = middlewareFunction;
//# sourceMappingURL=middleware.js.map